import React, { PureComponent } from "react";
import { Model } from "fineui-fix/src/model";
import { Watcher } from "fineui-fix/src/observer/watcher";
import { popTarget, pushTarget } from "fineui-fix/src/observer/dep";
import isFunction from "lodash/isFunction";
import each from "lodash/each";
import isObject from "lodash/isObject";

const targetStack = [];
function pushStore(_target) {
  if (_target) targetStack.push(_target);
  Model.target = _target;
}

function popStore() {
  Model.target = targetStack.pop();
}

export function inject(configFn, attrFn) {
  return function(ComponentClass) {
    class InjectClass extends PureComponent {
      constructor(props, context) {
        super(props, context);
        const config = isFunction(configFn) ? configFn(props) : configFn;
        this.store = new Model(config, props);
        this.watcher = new Watcher(
          this,
          () => {
            const {
              props: p,
              state,
              computed,
              _parent,
              _watchers,
              _computedWatchers,
              $$props,
              $$state,
              $$computed,
              $$context,
              ...others
            } = this.store;
            return (this.attrProps = attrFn
              ? attrFn(this.store, props)
              : { ...others });
          },
          () => {
            this.forceUpdate();
          }
        );
      }
      componentWillMount() {
        pushStore(this.store);
      }
      render() {
        return React.createElement(
          ComponentClass,
          Object.assign({}, this.attrProps, this.props)
        );
      }
      componentDidMount() {
        popStore();
      }
      componentWillReceiveProps(nextProps) {
        each(this.store.$$props, (prop, key) => {
          // 如果是数组，就刷新吧
          if (Array.isArray(nextProps[key])) {
            this.store.$$props[key] = [...nextProps[key]];
          } else {
            this.store.$$props[key] = nextProps[key];
          }
        });
      }
      componentWillUpdate() {
        pushStore(this.store);
      }
      componentDidUpdate() {
        popStore();
      }
      componentWillUnmount() {
        this.watcher.teardown();
        this.store.destroy();
      }
    }
    InjectClass.displayName =
      "Inject" + (ComponentClass.displayName || ComponentClass.name);
    return InjectClass;
  };
}

export default inject;
